"""The base Controller API

Provides the BaseController class for subclassing.
"""
from pylons.controllers import WSGIController
from pylons.templating import render_mako as render, pylons_globals

from ajax.model import meta

class BaseController(WSGIController):

    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # WSGIController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']
        try:
            return WSGIController.__call__(self, environ, start_response)
        finally:
            meta.Session.remove()


def render_def(template_name, name, **kwargs):
    """Directly render a <%def> defined in a Mako template.
    
        template_name - URI of the template.
        name - the name of the <%def> to invoke
        \**kwargs - keyword arguments will be sent directly to the def, in 
        addition to the regular Pylons namespace - names in \**kwargs
        take precedence.
        
    """
    globs = pylons_globals()

    if kwargs:
        globs = globs.copy()
        globs.update(kwargs)

    template = globs['app_globals'].mako_lookup.get_template(template_name).get_def(name)
    return template.render(**globs)
    